﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Vintagestory.API.Client;
using Vintagestory.Client.NoObf;
using Vintagestory.Client;
using Vintagestory.API.Config;
using Vintagestory.API.Common;
using Vintagestory.API.Server;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.MathTools;
using Vintagestory.API.Util;
using Vintagestory.GameContent;
using Vintagestory.API.Datastructures;
using ProtoBuf;

namespace nearsight
{
    class CommandsSystem : ModSystem
    {
        SettingsSync syncer;

        public override double ExecuteOrder()
        {
            return 0.31d;
        }

        public override bool ShouldLoad(EnumAppSide side)
        {
            return side == EnumAppSide.Server;
        }

        public override void StartServerSide(ICoreServerAPI api)
        {
            base.StartServerSide(api);

            syncer = api.ModLoader.GetModSystem<SettingsSync>();

            var parsers = api.ChatCommands.Parsers;
            api.ChatCommands.Create("nearsighted")
                .RequiresPrivilege(Privilege.chat)
                .BeginSubCommand("set")
                    .RequiresPrivilege(Privilege.chat)
                    .WithArgs(parsers.DoubleRange("distance", 0d, 1d), parsers.Double("strength"))
                    .HandleWith(setNearsight)
                    .EndSubCommand()
                .BeginSubCommand("clear")
                    .RequiresPrivilege(Privilege.chat)
                    .HandleWith(clearNearsight)
                    .EndSubCommand();
        }

        private TextCommandResult clearNearsight(TextCommandCallingArgs args)
        {
            IServerPlayer splayer = (args.Caller.Player as IServerPlayer);
            splayer.SetModData<float>("nearsightSettings.Distance", 0);
            splayer.SetModData<float>("nearsightSettings.Strength", 0);
            splayer.SetModData<bool>("nearsightSettings.Enabled", false);

            syncer.SyncSettings(args.Caller.Player as IServerPlayer);

            return TextCommandResult.Success();
        }

        private TextCommandResult setNearsight(TextCommandCallingArgs args)
        {
            NearsightednessSettings settings = new NearsightednessSettings((float)((double)args.Parsers[0].GetValue()), (float)((double)args.Parsers[1].GetValue()), true);


            IServerPlayer splayer = (args.Caller.Player as IServerPlayer);
            splayer.SetModData<float>("nearsightSettings.Distance", settings.MinBlurDistance);
            splayer.SetModData<float>("nearsightSettings.Strength", settings.BlurRadius);
            splayer.SetModData<bool>("nearsightSettings.Enabled", settings.BlurEnabled);

            syncer.SyncSettings(args.Caller.Player as IServerPlayer);

            return TextCommandResult.Success();
        }
    }
}
